﻿Imports BookingWebModel

Public Class AzureMonthViewStore
    Implements IMonthViewWriter, IMonthViewReader

    Private ReadOnly viewContainer As CloudBlobContainer

    Public Sub New(ByVal viewContainer As CloudBlobContainer)
        If viewContainer Is Nothing Then Throw New ArgumentNullException("viewContainer")

        Me.viewContainer = viewContainer
    End Sub

#Region "IMonthViewStore Members"

    Public Sub Disable(ByVal [date] As Date) Implements BookingWebModel.IMonthViewWriter.Disable
        Dim viewBlob = Me.GetViewBlob([date])
        Dim disabledDates() = viewBlob.DownloadItem()
        viewBlob.Upload(disabledDates.Union({[date]}).ToArray())
    End Sub

#End Region

#Region "IMonthViewReader Members"

    Public Function Read(ByVal year As Integer, ByVal month As Integer) As System.Collections.Generic.IEnumerable(Of String) Implements BookingWebModel.IMonthViewReader.Read
        Dim disabledDates() = Me.GetViewBlob(year, month).DownloadItem()
        Return (From d In disabledDates
         Select d.ToString("yyyy.MM.dd"))
    End Function
#End Region

    Private Function GetViewBlob(ByVal [date] As Date) As SerializerBlob(Of Date())
        Dim viewName = String.Format("view.month.{0}", [date].ToString("yyyyMM"))
        Return Me.GetViewBlob(viewName)
    End Function

    Private Function GetViewBlob(ByVal year As Integer, ByVal month As Integer) As SerializerBlob(Of Date())
        Dim viewName = String.Format("view.month.{0:D2}{1:D2}", year, month)
        Return Me.GetViewBlob(viewName)
    End Function

    Private Function GetViewBlob(ByVal viewName As String) As SerializerBlob(Of Date())
        Return New SerializerBlob(Of Date())(Me.viewContainer.GetBlobReference(viewName), Function() New Date() {})
    End Function
End Class