﻿Imports BookingDomainModel

Public Class AzureReservationRepository
    Implements IReservationRepository

    Private ReadOnly context As ReservationContext

    Public Sub New(ByVal context As ReservationContext)
        If context Is Nothing Then Throw New ArgumentNullException("context")

        Me.context = context
    End Sub

#Region "IReservationRoot Members"

    Public Sub AddReservation(ByVal reservation As ReservationAcceptedEvent) Implements BookingDomainModel.IReservationRepository.AddReservation
        Dim partitionKey = reservation.Date.Date.ToString("yyyyMMdd")
        Dim rowKey = reservation.Id.ToString()

        If Me.IsReplay(partitionKey, rowKey) Then Return

        Dim r =
            New ReservationTableEntity With {
                .PartitionKey = reservation.Date.Date.ToString("yyyyMMdd"),
                .RowKey = reservation.Id.ToString(),
                .Date = reservation.Date,
                .Name = reservation.Name,
                .Email = reservation.Email,
                .Quantity = reservation.Quantity}

        Me.context.AddReservation(r)
        Me.context.SaveChanges()
    End Sub

#End Region

    Private Function IsReplay(ByVal partitionKey As String, ByVal rowKey As String) As Boolean
        Try
            Return Me.context.Reservations.Where(Function(x) x.PartitionKey = partitionKey AndAlso x.RowKey = rowKey).AsEnumerable().Any()
        Catch e As DataServiceQueryException
            Dim ie = TryCast(e.InnerException, DataServiceClientException)
            If (ie IsNot Nothing) AndAlso (ie.StatusCode = 404) Then Return False
            Throw
        End Try
    End Function
End Class