﻿Imports BookingDomainModel
Imports BookingWebModel

Public Class CapacityGuard
    Implements IMessageConsumer(Of MakeReservationCommand), IDayViewReader

    Private ReadOnly capacityContainer As CloudBlobContainer
    Private ReadOnly channel As IChannel

    Public Sub New(ByVal capacityContainer As CloudBlobContainer, ByVal channel As IChannel)
        If capacityContainer Is Nothing Then Throw New ArgumentNullException("capacityContainer")
        If channel Is Nothing Then Throw New ArgumentNullException("channel")

        Me.capacityContainer = capacityContainer
        Me.channel = channel
    End Sub

    Public Function HasCapacity(ByVal reservation As MakeReservationCommand) As Boolean
        Return Me.GetCapacityBlob(reservation).DownloadItem().CanReserve(reservation.Quantity, reservation.Id)
    End Function

#Region "IMessageConsumer<MakeReservationCommand> Members"

    Public Sub Consume(ByVal message As MakeReservationCommand) Implements BookingDomainModel.IMessageConsumer(Of BookingDomainModel.MakeReservationCommand).Consume
        Dim blob = Me.GetCapacityBlob(message)
        Dim originalCapacity = blob.DownloadItem()

        Dim newCapacity = originalCapacity.Reserve(message.Quantity, message.Id)

        If Not newCapacity.Equals(originalCapacity) Then
            blob.Upload(newCapacity)
            If newCapacity.Remaining <= 0 Then
                Dim e = New SoldOutEvent(message.Date)
                Me.channel.Send(e)
            End If
        End If
    End Sub

#End Region

#Region "IDayViewReader Members"

    Public Function GetRemainingCapacity(ByVal [date] As Date) As Integer Implements BookingWebModel.IDayViewReader.GetRemainingCapacity
        Return Me.GetCapacityBlob([date]).DownloadItem().Remaining
    End Function

#End Region

    Private Function GetCapacityBlob(ByVal [date] As Date) As SerializerBlob(Of Capacity)
        Dim capacityName = CapacityGuard.GetCapacityName([date])
        Return New SerializerBlob(Of Capacity)(Me.capacityContainer.GetBlobReference(capacityName), Function() New Capacity(10))
    End Function

    Private Function GetCapacityBlob(ByVal reservation As MakeReservationCommand) As SerializerBlob(Of Capacity)
        Return Me.GetCapacityBlob(reservation.Date)
    End Function

    Private Shared Function GetCapacityName(ByVal [date] As Date) As String
        Return String.Format("date.{0}", [date].ToString("yyyyMMdd"))
    End Function
End Class