﻿Imports System.Runtime.Serialization.Formatters.Binary

Public Class SerializerBlob(Of T)
    Inherits CloudBlob
    Private ReadOnly createNewItem As Func(Of T)
    Private ReadOnly formatter As BinaryFormatter

    Public Sub New(ByVal cloudBlob As CloudBlob, ByVal initialValueFactory As Func(Of T))
        MyBase.New(cloudBlob)
        If initialValueFactory Is Nothing Then Throw New ArgumentNullException("initialValueFactory")

        Me.createNewItem = initialValueFactory
        Me.formatter = New BinaryFormatter
    End Sub

    Public Function DownloadItem() As T
        Dim s = New MemoryStream
        Try
            Me.DownloadToStream(s)
            s.Position = 0

            Return CType(Me.formatter.Deserialize(s), T)
        Catch e As StorageClientException
            If e.ErrorCode <> StorageErrorCode.BlobNotFound Then Throw

            Return Me.createNewItem()
        Finally
            s.Dispose()
        End Try
    End Function

    Public Sub Upload(ByVal item As T)
        Using s = New MemoryStream
            Me.formatter.Serialize(s, item)
            s.Position = 0

            Dim options = Me.CreateBlobRequestOptions()
            Me.UploadFromStream(s, options)
        End Using
    End Sub

    Private Function CreateBlobRequestOptions() As BlobRequestOptions
        Dim etag = Me.Properties.ETag
        Dim options = New BlobRequestOptions
        SerializerBlob(Of T).SetAccessCondition(etag, options)
        Return options
    End Function

    Private Shared Sub SetAccessCondition(ByVal etag As String, ByVal options As BlobRequestOptions)
        options.AccessCondition = If(etag Is Nothing, AccessCondition.IfNoneMatch("*"), AccessCondition.IfMatch(etag))
    End Sub
End Class