﻿Public Class TableStorageMailClient
    Private ReadOnly context As MailContext

    Public Sub New(ByVal context As MailContext)
        If context Is Nothing Then Throw New ArgumentNullException("context")

        Me.context = context
    End Sub

    Public Sub Send(ByVal msg As MailMessage)
        Dim time = Date.UtcNow

        Dim partitionKey = time.Date.ToString("yyyyMMdd")
        Dim rowKey = Guid.NewGuid().ToString("N")
        If Me.IsReplay(partitionKey, rowKey) Then Return

        Dim m = New MailMessageTableEntity With
                {.PartitionKey = partitionKey,
                 .RowKey = rowKey,
                 .Body = msg.Body,
                 .Recipients = msg.To.Select(Function(a) a.ToString()).Aggregate(Function(x, y) x & "; " & y),
                 .Sender = msg.From.ToString(),
                 .Subject = msg.Subject}

        Me.context.AddMailMessage(m)
        Me.context.SaveChanges()
    End Sub

    Private Function IsReplay(ByVal partitionKey As String, ByVal rowKey As String) As Boolean
        Try
            Return Me.context.MailMessages.Where(Function(x) x.PartitionKey = partitionKey AndAlso x.RowKey = rowKey).AsEnumerable().Any()
        Catch e As DataServiceQueryException
            Dim ie = TryCast(e.InnerException, DataServiceClientException)
            If (ie IsNot Nothing) AndAlso (ie.StatusCode = 404) Then Return False
            Throw
        End Try
    End Function
End Class