﻿<Serializable()>
Public Class Capacity
    Implements IEquatable(Of Capacity)

    Private ReadOnly _remaining As Integer
    Private ReadOnly acceptedReservations As IEnumerable(Of Guid)

    Public Sub New(ByVal remaining As Integer, ByVal ParamArray acceptedReservations() As Guid)
        Me._remaining = remaining
        Me.acceptedReservations = acceptedReservations
    End Sub

    Public ReadOnly Property Remaining As Integer
        Get
            Return Me._remaining
        End Get
    End Property

    Public Function CanReserve(ByVal quantity As Integer, ByVal id As Guid) As Boolean
        If Me.IsReplay(id) Then Return True
        Return Me._remaining >= quantity
    End Function

    Private Function IsReplay(ByVal id As Guid) As Boolean
        Return Me.acceptedReservations.Contains(id)
    End Function

    Public Overloads Overrides Function Equals(ByVal obj As Object) As Boolean
        Dim other = TryCast(obj, Capacity)
        If other IsNot Nothing Then Return Me.Equals(other)
        Return MyBase.Equals(obj)
    End Function

    Public Overrides Function GetHashCode() As Integer
        Return Me.acceptedReservations.Select(Function(g) g.GetHashCode()).Aggregate(Function(x, y) x Xor y) Xor Me.Remaining
    End Function

    Public Function Reserve(ByVal quantity As Integer, ByVal id As Guid) As Capacity
        If Not Me.CanReserve(quantity, id) Then Throw New ArgumentOutOfRangeException("quantity", "The quantity must be less than or equal to the remaining quantity.")

        If Me.IsReplay(id) Then Return Me

        Return New Capacity(Me.Remaining - quantity, Me.acceptedReservations.Concat({id}).ToArray())
    End Function

#Region "IEquatable<Capacity> Members"

    Public Overloads Function Equals(ByVal other As Capacity) As Boolean Implements System.IEquatable(Of Capacity).Equals
        If other Is Nothing Then Return False

        Return (Me.Remaining = other.Remaining) AndAlso Not Me.acceptedReservations.Except(other.acceptedReservations).Any()
    End Function

#End Region
End Class