﻿Imports Moq
Imports BookingDomainModel

Public Class CompositeMessageConsumerFacts
    <Theory(), AutoDomainData()>
    Public Sub SutIsMessageConsumer(ByVal sut As CompositeMessageConsumer(Of Object))
        Assert.IsAssignableFrom(Of IMessageConsumer(Of Object))(sut)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub ConsumersIsCorrect(<Frozen()> ByVal consumers() As IMessageConsumer(Of Object),
                                  ByVal sut As CompositeMessageConsumer(Of Object))
        Assert.True(consumers.SequenceEqual(sut.Consumers))
    End Sub

    <Fact()>
    Public Sub ConsumeConsumesAllConsumers()
        'Fixture setup
        Dim fixture = New Fixture().Customize(New DomainCustomization())
        Dim mocks = fixture.CreateMany(Of Mock(Of IMessageConsumer(Of Object)))().ToList()
        fixture.Inject(mocks.Select(Function(m) m.Object).ToArray())

        Dim message = fixture.CreateAnonymous(Of Object)()

        Dim sut = fixture.CreateAnonymous(Of CompositeMessageConsumer(Of Object))()
        'Exercise system
        sut.Consume(message)
        'Verify outcome
        mocks.ForEach(Sub(m) m.Verify(Sub(c) c.Consume(message)))
        'Teardown
    End Sub
End Class