﻿Imports Moq
Imports BookingDomainModel

Public Class CompositeReservationRootFacts
    <Theory(), AutoDomainData()>
    Public Sub SutIsReservationRoot(ByVal sut As CompositeReservationRoot)
        Assert.IsAssignableFrom(Of IReservationRepository)(sut)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub RootsIsCorrect(<Frozen()> ByVal roots() As IReservationRepository, ByVal sut As CompositeReservationRoot)
        Assert.True(roots.SequenceEqual(sut.Roots))
    End Sub

    <Fact()>
    Public Sub AddReservationAddsReservationToAllComposedRoots()
        'Fixture setup
        Dim fixture = New Fixture().Customize(New DomainCustomization())
        Dim rootMocks = fixture.CreateMany(Of Mock(Of IReservationRepository))().ToList()
        fixture.Inject(rootMocks.Select(Function(m) m.Object).ToArray())

        Dim reservation = fixture.CreateAnonymous(Of ReservationAcceptedEvent)()

        Dim sut = fixture.CreateAnonymous(Of CompositeReservationRoot)()
        'Exercise system
        sut.AddReservation(reservation)
        'Verify outcome
        rootMocks.ForEach(Sub(m) m.Verify(Sub(r) r.AddReservation(reservation)))
        'Teardown
    End Sub
End Class