﻿Imports Moq
Imports BookingDomainModel

Public Class ConditionalMessageConsumerFacts
    <Theory(), AutoDomainData()>
    Public Sub SutIsMessageConsumer(ByVal sut As ConditionalMessageConsumer(Of Object))
        Assert.IsAssignableFrom(Of IMessageConsumer(Of Object))(sut)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub FirstIsCorrect(ByVal first As IMessageConsumer(Of Object),
                              ByVal second As IMessageConsumer(Of Object),
                              ByVal condition As Func(Of Object, Boolean)
                              )
        Dim sut = ConditionalMessageConsumerFacts.CreateSut(first, second, condition)
        Dim result = sut.First
        Assert.Equal(first, result)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SecondIsCorrect(ByVal first As IMessageConsumer(Of Object),
                               ByVal second As IMessageConsumer(Of Object),
                               ByVal condition As Func(Of Object, Boolean)
                               )
        Dim sut = ConditionalMessageConsumerFacts.CreateSut(first, second, condition)
        Dim result = sut.Second
        Assert.Equal(second, result)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub ConditionIsCorrect(<Frozen()> ByVal condition As Func(Of Object, Boolean),
                                  ByVal sut As ConditionalMessageConsumer(Of Object))
        Dim result = sut.Condition
        Assert.Equal(condition, result)
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub FirstIsInvokedWhenConditionIsTrue(ByVal firstMock As Mock(Of IMessageConsumer(Of Object)),
                                                 ByVal secondStub As Mock(Of IMessageConsumer(Of Object)),
                                                 ByVal message As Object)
        Dim sut = ConditionalMessageConsumerFacts.CreateSut(firstMock.Object, secondStub.Object, Function(m) True)
        sut.Consume(message)
        firstMock.Verify(Sub(c) c.Consume(message))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SecondIsNotInvokedWhenConditionIsTrue(ByVal firstStub As Mock(Of IMessageConsumer(Of Object)),
                                                     ByVal secondMock As Mock(Of IMessageConsumer(Of Object)),
                                                     ByVal message As Object
                                                     )
        Dim sut = ConditionalMessageConsumerFacts.CreateSut(firstStub.Object, secondMock.Object, Function(m) True)
        sut.Consume(message)
        secondMock.Verify(Sub(c) c.Consume(It.IsAny(Of Object)()), Times.Never())
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub SecondIsInvokedWhenConditionIsFalse(ByVal firstStub As Mock(Of IMessageConsumer(Of Object)),
                                                   ByVal secondMock As Mock(Of IMessageConsumer(Of Object)),
                                                   ByVal message As Object
                                                   )
        Dim sut = ConditionalMessageConsumerFacts.CreateSut(firstStub.Object, secondMock.Object, Function(m) False)
        sut.Consume(message)
        secondMock.Verify(Sub(c) c.Consume(message))
    End Sub

    <Theory(), AutoDomainData()>
    Public Sub FirstIsNotInvokedWhenConditionIsFalse(ByVal firstMock As Mock(Of IMessageConsumer(Of Object)),
                                                     ByVal secondStub As Mock(Of IMessageConsumer(Of Object)),
                                                     ByVal message As Object
                                                     )
        Dim sut = ConditionalMessageConsumerFacts.CreateSut(firstMock.Object, secondStub.Object, Function(m) False)
        sut.Consume(message)
        firstMock.Verify(Sub(c) c.Consume(It.IsAny(Of Object)()), Times.Never())
    End Sub

    Private Shared Function CreateSut(Of T)(ByVal first As IMessageConsumer(Of T),
                                            ByVal second As IMessageConsumer(Of T),
                                            ByVal condition As Func(Of T, Boolean)
                                            ) As ConditionalMessageConsumer(Of T)
        Return New ConditionalMessageConsumer(Of T)(condition, first, second)
    End Function
End Class