﻿Imports Moq
Imports BookingWebModel
Imports BookingDomainModel

Public Class HomeControllerFacts
    <Theory(), AutoWebData()>
    Public Sub SutIsController(ByVal sut As HomeController)
        Assert.IsAssignableFrom(Of IController)(sut)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub IndexReturnsCorrectModel(<Frozen()> ByVal readerStub As Mock(Of IMonthViewReader),
                                        ByVal dates() As String,
                                        ByVal sut As HomeController)
        Dim start = Date.Now
        readerStub.Setup(Function(r) r.Read(
                             It.Is(Of Integer)(Function(y) start.Year <= y AndAlso y <= Date.Now.Year),
                             It.Is(Of Integer)(Function(m) start.Month <= m AndAlso m <= Date.Now.Month))).Returns(dates)
        Dim result = sut.Index()
        Assert.Equal(dates, result.ViewData.Model)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub NewBookingReturnsInstance(ByVal sut As HomeController,
                                         ByVal year As Integer,
                                         ByVal month As Integer,
                                         ByVal day As Integer)
        Dim result = sut.NewBooking(year, month, day)
        Assert.NotNull(result)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub NewBookingReturnsCorrectTypeOfModel(ByVal sut As HomeController,
                                                   ByVal year As Integer,
                                                   ByVal month As Integer,
                                                   ByVal day As Integer)
        Dim result = sut.NewBooking(year, month, day)
        Assert.IsAssignableFrom(Of BookingViewModel)(result.ViewData.Model)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub NewBookingReturnsModelWithCorrectDate(ByVal sut As HomeController,
                                                     ByVal year As Integer,
                                                     ByVal month As Integer,
                                                     ByVal day As Integer)
        Dim result = sut.NewBooking(year, month, day)
        Dim vm = Assert.IsAssignableFrom(Of BookingViewModel)(result.ViewData.Model)
        Assert.Equal(New Date(year, month, day), vm.Date)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub NewBookingReturnsModelWithCorrectRemaining(<Frozen()> ByVal readerStub As Mock(Of IDayViewReader),
                                                          ByVal remaining As Integer,
                                                          ByVal sut As HomeController,
                                                          ByVal year As Integer,
                                                          ByVal month As Integer,
                                                          ByVal day As Integer)
        readerStub.Setup(Function(r) r.GetRemainingCapacity(New Date(year, month, day))).Returns(remaining)
        Dim result = sut.NewBooking(year, month, day)
        Dim vm = Assert.IsAssignableFrom(Of BookingViewModel)(result.ViewData.Model)
        Assert.Equal(remaining, vm.Remaining)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub NewBookingPostReturnsInstance(ByVal sut As HomeController, ByVal model As BookingViewModel)
        Dim result = sut.NewBooking(model)
        Assert.NotNull(result)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub NewBookingPostReturnsCorrectViewName(ByVal sut As HomeController, ByVal model As BookingViewModel)
        Dim result = sut.NewBooking(model)
        Assert.Equal("BookingReceipt", result.ViewName)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub NewBookingPostReturnsCorrectTypeOfModel(ByVal sut As HomeController, ByVal model As BookingViewModel)
        Dim result = sut.NewBooking(model)
        Assert.IsAssignableFrom(Of BookingViewModel)(result.ViewData.Model)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub NewBookingPostReturnsCorrectModel(ByVal sut As HomeController, ByVal model As BookingViewModel)
        Dim result = sut.NewBooking(model)
        Assert.Equal(model, result.ViewData.Model)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub NewBookingPostCorrectlySendsOnChannel(<Frozen()> ByVal channelMock As Mock(Of IChannel),
                                                     ByVal sut As HomeController,
                                                     ByVal model As BookingViewModel)
        sut.NewBooking(model)
        Dim expected = model.AsSource().OfLikeness(Of MakeReservationCommand)().Without(Function(d) d.Id)
        channelMock.Verify(Sub(c) c.Send(expected))
    End Sub

    <Theory(), AutoWebData()>
    Public Sub DisabledDaysReturnsCorrectResult(<Frozen()> ByVal readerStub As Mock(Of IMonthViewReader),
                                                ByVal disableDays() As String,
                                                ByVal sut As HomeController,
                                                ByVal year As Integer,
                                                ByVal month As Integer)
        readerStub.Setup(Function(r) r.Read(year, month)).Returns(disableDays)
        Dim result = sut.DisabledDays(year, month)
        Assert.Equal(disableDays, result.Data)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub DisabledDaysReturnsCorrectJsonBehavior(ByVal sut As HomeController,
                                                      ByVal year As Integer,
                                                      ByVal month As Integer)
        Dim result = sut.DisabledDays(year, month)
        Assert.Equal(JsonRequestBehavior.AllowGet, result.JsonRequestBehavior)
    End Sub
End Class