﻿Imports BookingWebModel

Public Class MonthViewModelFacts
    <Theory(), AutoWebData()>
    Public Sub DaysIsInstance(ByVal sut As MonthViewModel)
        Dim result As IEnumerable(Of DayViewModel) = sut.Days
        Assert.NotNull(result)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub DaysIsStable(ByVal sut As MonthViewModel)
        Dim expected = sut.Days
        Assert.Equal(expected, sut.Days)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub DaysIsCorrectWhenUsingGreedyConstructor(<Frozen()> ByVal days() As DayViewModel, <Greedy()> ByVal sut As MonthViewModel)
        Assert.True(days.SequenceEqual(sut.Days))
    End Sub

    <Theory(), AutoWebData()>
    Public Sub MonthIsCorrectWhenUsingGreedyConstructor(<Frozen()> ByVal month As Integer, <Greedy()> ByVal sut As MonthViewModel)
        Assert.Equal(month, sut.Month)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub YearIsCorrectWhenUsingGreedyConstructor(<Frozen()> ByVal year As Integer, <Greedy()> ByVal sut As MonthViewModel)
        Assert.Equal(year, sut.Year)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub DayCountIsCorrectWhenUsingDateBasedConstructor(<Frozen()> ByVal [date] As Date, <Modest()> ByVal sut As MonthViewModel)
        Dim dayCount = New GregorianCalendar().GetDaysInMonth([date].Year, [date].Month)
        Assert.Equal(dayCount, sut.Days.Count())
    End Sub

    <Theory(), AutoWebData()>
    Public Sub DaysHaveCorrectNumbersWhenUsingDateBasedConstructor(<Frozen()> ByVal [date] As Date, <Modest()> ByVal sut As MonthViewModel)
        Dim dayCount = New GregorianCalendar().GetDaysInMonth([date].Year, [date].Month)
        Assert.True(Enumerable.Range(1, dayCount).SequenceEqual(sut.Days.Select(Function(m) m.Day)))
    End Sub

    <Theory(), AutoWebData()>
    Public Sub MonthIsCorrectWhenUsingDateBasedConstructor(<Frozen()> ByVal [date] As Date, <Modest()> ByVal sut As MonthViewModel)
        Assert.Equal([date].Month, sut.Month)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub YearIsCorrectWhenUsingDateBasedConstructor(<Frozen()> ByVal [date] As Date, <Modest()> ByVal sut As MonthViewModel)
        Assert.Equal([date].Year, sut.Year)
    End Sub

    <Theory(), AutoWebData()>
    Public Sub DisplayNameIsCorrect(ByVal sut As MonthViewModel)
        Dim aDayInTheMonth = New Date(sut.Year, sut.Month, 1)
        Dim expected = aDayInTheMonth.ToString("MMMM yyyy")
        Assert.Equal(Of String)(expected, sut.DisplayName)
    End Sub
End Class