﻿Imports BookingWebModel
Imports BookingDomainModel
Imports BookingAzureServices

Public Class BookingWebRoleWindsorInstaller
    Implements IWindsorInstaller

#Region "IWindsorInstaller Members"

    Public Sub Install(ByVal container As IWindsorContainer, ByVal store As IConfigurationStore) Implements Castle.MicroKernel.Registration.IWindsorInstaller.Install
        container.Register(AllTypes.FromAssemblyContaining(Of HomeController)().BasedOn(Of IController)().Configure(Function(r) r.LifeStyle.PerWebRequest))

        container.Register(Component.For(Of IMonthViewReader)().ImplementedBy(Of AzureMonthViewStore)().ServiceOverrides(New With {Key .viewContainer = "viewContainer"}))
        container.Register(Component.For(Of IDayViewReader)().ImplementedBy(Of CapacityGuard)().ServiceOverrides(New With {Key .viewContainer = "capacityContainer"}))
        container.Register(Component.For(Of IChannel)().ImplementedBy(Of AzureChannel)().LifeStyle.PerWebRequest)
        container.Register(Component.For(Of CloudQueue)().UsingFactoryMethod(
                           Function(k)
                               Dim queueName = "messagequeue"
                               Dim client = k.Resolve(Of CloudQueueClient)()
                               Dim queue = client.GetQueueReference(queueName)
                               queue.CreateIfNotExist()
                               Return queue
                           End Function).LifeStyle.PerWebRequest)
        container.Register(Component.For(Of CloudQueueClient)().UsingFactoryMethod(
                           Function(k) k.Resolve(Of CloudStorageAccount)().CreateCloudQueueClient()).LifeStyle.PerWebRequest)
        container.Register(Component.For(Of CloudBlobContainer)().UsingFactoryMethod(
                           Function(k)
                               Dim c = k.Resolve(Of CloudBlobClient)().GetContainerReference("capacity")
                               c.CreateIfNotExist()
                               Return c
                           End Function).Named("capacityContainer"))
        container.Register(Component.For(Of CloudBlobContainer)().UsingFactoryMethod(
                           Function(k)
                               Dim c = k.Resolve(Of CloudBlobClient)().GetContainerReference("view")
                               c.CreateIfNotExist()
                               Return c
                           End Function).Named("viewContainer"))
        container.Register(Component.For(Of CloudBlobClient)().UsingFactoryMethod(
                           Function(k) k.Resolve(Of CloudStorageAccount)().CreateCloudBlobClient()))
        container.Register(Component.For(Of CloudStorageAccount)().UsingFactoryMethod(
                           Function() CloudStorageAccount.FromConfigurationSetting("DataConnectionString")).LifeStyle.PerWebRequest)
    End Sub

#End Region
End Class