﻿Imports System.Diagnostics
Imports BookingDomainModel

Public Class AzureQueueMessageProcessor
    Private container As IWindsorContainer

    Public Sub New(ByVal container As IWindsorContainer)
        Me.container = container
    End Sub

    Public Sub Run(ByVal token As CancellationToken)
        Dim queue = Me.container.Resolve(Of CloudQueue)()

        Do While Not token.IsCancellationRequested
            Trace.WriteLine("Polling for message.", "Verbose")
            Me.PollForMessage(queue)
        Loop
    End Sub

    Public Sub PollForMessage(ByVal queue As CloudQueue)
        Dim message = queue.GetMessage()
        If message Is Nothing Then
            Thread.Sleep(500)
            Return
        End If

        Try
            Me.Handle(message)
            queue.DeleteMessage(message)
        Catch e As Exception
            If e.IsUnsafeToSuppress() Then Throw
            Trace.TraceError(e.ToString())
        End Try
    End Sub

    Public Sub Handle(ByVal message As CloudQueueMessage)
        If message Is Nothing Then Throw New ArgumentNullException("message")
        Dim formatter = New BinaryFormatter
        Using s = New MemoryStream(message.AsBytes)
            Dim consumable As Object = formatter.Deserialize(s)
            Trace.TraceInformation("Received {0} message.", CObj(consumable))

            Dim consumerType = GetType(IMessageConsumer(Of )).MakeGenericType(consumable.GetType())
            Dim consumer As Object = Me.container.Resolve(consumerType)
            consumer.Consume(consumable)
        End Using
    End Sub
End Class