﻿Imports BookingAzureServices
Imports BookingDomainModel
Imports BookingWebModel
Imports SmtpServices

Public Class BookingWorkerRoleWindsorInstaller
    Implements IWindsorInstaller

#Region "IWindsorInstaller Members"

    Public Sub Install(ByVal container As IWindsorContainer, ByVal store As IConfigurationStore) Implements Castle.MicroKernel.Registration.IWindsorInstaller.Install
        'container.Kernel.Resolver.AddSubResolver(New CollectionResolver(container.Kernel))

        'container.Register(AllTypes
        '.FromAssemblyContaining(Of MakeNewReservationConsumer)()
        '.BasedOn(GetType(IMessageConsumer(Of )))
        '.WithService.AllInterfaces())
        container.Register(Component.
                           For(Of IMessageConsumer(Of MakeReservationCommand))().
                           UsingFactoryMethod(Of ConditionalMessageConsumer(Of MakeReservationCommand))(
                               Function(k)
                                   Dim guard = k.Resolve(Of CapacityGuard)()
                                   Dim first = New CompositeMessageConsumer(Of MakeReservationCommand)(guard,
                                                                                                       k.Resolve(Of ReservationWriter)(),
                                                                                                       k.Resolve(Of AcceptingReservationConsumer)())
                                   Dim second = k.Resolve(Of RejectingReservationConsumer)()
                                   Return New ConditionalMessageConsumer(Of MakeReservationCommand)(AddressOf guard.HasCapacity, first, second)
                               End Function))

        container.Register(Component.For(Of IMessageConsumer(Of ReservationAcceptedEvent))().ImplementedBy(Of AcceptedReserverationMailer)())
        container.Register(Component.For(Of IMessageConsumer(Of ReservationRejectedEvent))().ImplementedBy(Of RejectedReservationMailer)())
        container.Register(Component.For(Of IMessageConsumer(Of SoldOutEvent))().ImplementedBy(Of MonthViewModelUpdater)())
        container.Register(Component.For(Of ReservationWriter)())
        container.Register(Component.For(Of AcceptingReservationConsumer)())
        container.Register(Component.For(Of RejectingReservationConsumer)())
        container.Register(Component.For(Of IReservationRepository)().ImplementedBy(Of CompositeReservationRoot)().ServiceOverrides(New With {Key .roots = {"azureReservationRoot"}})) ', "publishingReserverationRoot"

        'container.Register(AllTypes
        '    .FromAssemblyContaining(Of MonthViewModelUpdater)()
        '    .BasedOn(typeof(IMessageConsumer(Of )))
        '    .WithService.AllInterfaces())

        container.Register(Component.For(Of IReservationRepository)().ImplementedBy(Of AzureReservationRepository)().Named("azureReservationRoot"))
        container.Register(Component.For(Of ReservationContext)())
        container.Register(Component.For(Of MailContext)())
        container.Register(Component.For(Of IChannel)().ImplementedBy(Of AzureChannel)())
        container.Register(Component.For(Of CapacityGuard)().ServiceOverrides(New With {Key .capacityContainer = "capacityContainer"}))
        container.Register(Component.For(Of IMonthViewWriter)().ImplementedBy(Of AzureMonthViewStore)().ServiceOverrides(New With {Key .viewContainer = "viewContainer"}))

        container.Register(Component.For(Of CloudQueue)().UsingFactoryMethod(Function(k)
                                                                                 Dim queueName = "messagequeue"
                                                                                 Dim client = k.Resolve(Of CloudQueueClient)()
                                                                                 Dim queue = client.GetQueueReference(queueName)
                                                                                 queue.CreateIfNotExist()
                                                                                 Return queue
                                                                             End Function))
        container.Register(Component.For(Of CloudBlobContainer)().UsingFactoryMethod(Function(k)
                                                                                         Dim c = k.Resolve(Of CloudBlobClient)().GetContainerReference("capacity")
                                                                                         c.CreateIfNotExist()
                                                                                         Return c
                                                                                     End Function).Named("capacityContainer"))
        container.Register(Component.For(Of CloudBlobContainer)().UsingFactoryMethod(Function(k)
                                                                                         Dim c = k.Resolve(Of CloudBlobClient)().GetContainerReference("view")
                                                                                         c.CreateIfNotExist()
                                                                                         Return c
                                                                                     End Function).Named("viewContainer"))
        container.Register(Component.For(Of CloudQueueClient)().UsingFactoryMethod(Function(k) k.Resolve(Of CloudStorageAccount)().CreateCloudQueueClient()))
        container.Register(Component.For(Of CloudBlobClient)().UsingFactoryMethod(Function(k) k.Resolve(Of CloudStorageAccount)().CreateCloudBlobClient()))
        container.Register(Component.For(Of CloudStorageAccount)().UsingFactoryMethod(Function() CloudStorageAccount.FromConfigurationSetting("DataConnectionString")))

        container.Register(Component.For(Of IReservationRepository)().ImplementedBy(Of PublishingReserverationRoot)().Named("publishingReserverationRoot"))

        'container.Register(Component.For(Of Action(Of MailMessage))().UsingFactoryMethod(Of Action(Of MailMessage))(Function(k) AddressOf k.Resolve(Of SmtpClient)().Send))
        'container.Register(Component.For(Of SmtpClient)().UsingFactoryMethod(Function()
        '                                                                         Dim smtpServerAddress = RoleEnvironment.GetConfigurationSettingValue("SmtpServerAddress")
        '                                                                         Dim smtpUserName = RoleEnvironment.GetConfigurationSettingValue("SmtpUserName")
        '                                                                         Dim smtpPassword = RoleEnvironment.GetConfigurationSettingValue("SmtpPassword")
        '                                                                         Dim client = New SmtpClient(smtpServerAddress)
        '                                                                         client.Credentials = New NetworkCredential(smtpUserName, smtpPassword)
        '                                                                         Return client
        '                                                                     End Function))

        container.Register(Component.For(Of Action(Of MailMessage))().
                           UsingFactoryMethod(Of Action(Of MailMessage))(Function(k) AddressOf k.Resolve(Of TableStorageMailClient)().Send))
        container.Register(Component.For(Of TableStorageMailClient)())

        container.Register(Component.For(Of MailAddress)().UsingFactoryMethod(
                           Function()
                               Dim senderAddress = RoleEnvironment.GetConfigurationSettingValue("EmailSenderSmtpAddress")
                               Dim senderDisplayName = RoleEnvironment.GetConfigurationSettingValue("EmailSenderDisplayName")
                               Return New MailAddress(senderAddress, senderDisplayName)
                           End Function))
    End Sub

#End Region
End Class